import std/[os, strutils]
import nimcrypto/sysrand
import dictionary


const packed = staticRead("../data/dictionary.pack")


proc genPassphrase(dict: Dictionary, length, dictSize: int): string =
  var rands = newSeq[uint64](length)
  discard randomBytes(rands)

  var words: seq[string]
  for r in rands:
    let i = r mod dictSize.uint64
    words.add(dict[i].strip())
  result = words.join(" ")


const help = """Usage:
  passphrase [LENGTH] [DICTSIZE]

Defaults to length of 4 and dictionary size of 25,000."""


proc parseInput(dictLen: int): (int, int) =
  let params = commandLineParams()
  if "-h" in params or "--help" in params:
    echo help
    quit(0)

  var
    length = 4
    dictSize = 25_000

  if params.len > 0:
    try:
      length = parseInt(params[0])
    except ValueError:
      quit('"' & params[0] & "\" is not a valid passphrase length.", 1)

  if params.len > 1:
    try:
      dictSize = parseInt(params[1])
      if dictSize < 100 or dictSize > dictLen:
        quit("Dictionary size must be between 100 and " & $dictLen, 1)
    except ValueError:
      quit('"' & params[1] & "\" is not a valid dictionary size.", 1)

  result = (length, dictSize)


when isMainModule:
  let dict = packed.unpack()
  let (length, dictSize) = parseInput(dict.len)
  echo genPassphrase(dict, length, dictSize)
